-- Function Line Grapher V0.5b
-- Andrew Baron 1996
-- Expression evaluation code is by David Cuny, all other code by Andrew Baron

without type_check
include graphics.e
include get.e

global integer pars    -- current character being parsed in string
integer tmp
constant KEYBOARD=0
constant SCREEN=1

if graphics_mode(3) then
   puts(SCREEN, "Get A VGA Card!\n")
   abort(1)
end if

sequence xmin, xmax, ymin, ymax, Scale, Center, Command, plot, stepx, stepy, detail, plotbk
atom QUIT, chk, Y, X, prevX, prevY, count

stepx={1, 1}      --Steps to take when drawing the graph.
stepy={1, 1}
detail={1, .01}   --Detail level, smaller is more.  Don't go negative.
QUIT = 1          --Quit when QUIT is not 1
count=3           --Positon counter for displaying the X-Int.
plot="2*X-3\n"    --Default graph

puts(SCREEN, "               Welcome to Andrew's Function Line Grapher V 0.5b\n\n")
puts(SCREEN, "Enter the minimum and maximums for the X and Y axis.  Be careful not\n")
puts(SCREEN, "to have the minimum greater  than the maximum.   If this is the case\n")
puts(SCREEN, "then they will be reset to -10 and 10.\n\n")

puts(SCREEN, "XMin:")      --Get the Min/Max for X and Y
   xmin = get(KEYBOARD)
puts(SCREEN, "\nXMax:")
   xmax = get(KEYBOARD)
puts(SCREEN, "\nYMin:")
   ymin = get(KEYBOARD)
puts(SCREEN, "\nYMax:")
   ymax = get(KEYBOARD)

if xmin[2] > xmax[2] then     --Make sure xmin < xmax
   xmin[2] = -10
   xmax[2] = 10
   puts(SCREEN, "\nWARNING: X-axis has been reset to -10 -> 10.")
   tmp = wait_key()
end if

if ymin[2] > ymax[2] then     --Make sure ymin < ymax
   ymin[2] = -10
   ymax[2] = 10
   puts(SCREEN, "\nWARNING: Y-axis has been reset to -10 -> 10.")
   tmp = wait_key()
end if

--This is the beginning of David Cuny's Code.

--Because this is not my own code I did not change it all that much.  Just
--added the visual warning for division by zero.

function math( atom a, integer op, atom b )
 -- perform operation OP with A and B

 if op = '+' then
  return a + b
 elsif op = '-' then
  return a - b
 elsif op = '*' then
  return a * b
 elsif op = '/' then
  if b = 0 then
  puts(SCREEN, "You can't divide that by zero!\n")
  tmp = get(KEYBOARD)
   return 0
  else
   return a / b
  end if
 elsif op = '^' then
  return power( a, b )
 elsif op = '=' then
  return b
 elsif op = 0 then
   return a
 else
   puts(SCREEN, "\nERROR - Operand not valid\n")
   tmp = wait_key()
  return 1
 end if
end function


function isNumber( integer i )

 -- return true if character is part of a number

 if i = '.' then
  return -1 -- decimal place
 else
  return i >= '0' and i <= '9' -- true if ranges from '0' to '9'
 end if

end function


function isOp( integer i )

 -- return true if character is an operand

 return i = '+' or i = '-' or i = '*' or i = '/' or i = '^'

end function


function parse1( sequence s )
 integer op, decimal, sign
 atom accum, num

 op = '='    -- first operation is assignment to accumulator
 accum = 0   -- initialize accumulator
 sign = 1    -- unary minus
 while pars <= length(s) do -- scan

  if s[pars] = '(' then  -- left paren
   pars=pars+ 1  -- move past the paren
    num = parse1( s )  -- recurse
    accum = math( accum, op, num )  -- accumulate
  elsif s[pars] = ')' then
   return accum  -- return parsed expression
  elsif s[pars] = '-' then
   -- handle unary minus
   if pars= 1 then -- first character in string
    sign = -1 -- unary minus
   elsif isOp( s[pars-1] ) then -- prior character was op
    sign = -1 -- unary minus
   else
    op = s[pars] -- operand
   end if
  elsif isOp( s[pars] ) then
   op = s[pars] -- assign operand to pending operation
  elsif s[pars] = 'X' then
   accum = math( accum, op, X * sign ) -- variable
   sign = 1 -- clear the sign
  elsif isNumber( s[pars] ) then  -- number
   num = 0  -- clear accumulator
   decimal = 0 -- no decimal place
   while isNumber( s[pars] ) do  -- while it's a number
    if s[pars] = '.' then
     decimal = 1
    else
     num = ( num * 10 ) + ( s[pars] - '0' ) -- accumulate
     decimal = decimal * 10 -- decimal place
    end if
   pars=pars+ 1 -- increment pointer
   end while

   if decimal > 0 then  -- was there a decimal place?
    num = num / decimal -- divide by the decimal amount
   end if

  pars=pars- 1 -- back up one character (we went too far)

   num = num * sign -- add sign
   sign = 1 -- clear sign

   if s[pars+1] = 'X' then -- handle stuff like 3X
   pars=pars+ 1 -- move past X
    accum = math( accum, op, num * X ) -- include in operation
   else
    accum = math( accum, op, num ) -- perform pending operation
   end if
  else
      --  just continue
  end if

 pars=pars+ 1 -- increment to the next character

 end while

 return accum

end function


function parse( sequence s )

 -- main parsing function call
pars= 1 -- set pointer to first character
 return parse1( s ) -- parse expression

end function

atom result

--This is the end of David Cuny's Code.

tmp = graphics_mode(18)

procedure initial()     --Initialization function

X = xmin[2]             --Make xmin[2]/ymin[2] look better.
Y = ymin[2]

if xmax[2] - xmin[2] = 0 then  --look for division by zero
   puts(SCREEN, "\nERROR - Attempt to divide by zero:Xmax/Xmin is invalid - Set to 10/-10\n")
   xmin[2] = -10
   xmax[2] = 10
   tmp = wait_key()
end if

if ymax[2] - ymin[2] = 0 then  --look for division by zero
   puts(SCREEN, "\nERROR - Attempt to divide by zero:Ymax/Ymin is invalid - Set to 10/-10\n")
   ymin[2] = -10
   ymax[2] = 10
   tmp = wait_key()
end if

position (27,1)
printf(SCREEN, "xmin:%5.2f xmax:%5.2f stepx:%5.2f", {xmin[2], xmax[2], stepx[2]})
position (28,1)
printf(SCREEN, "ymin:%5.2f ymax:%5.2f stepy:%5.2f", {ymin[2], ymax[2], stepy[2]})
if compare(plot, "") = 0 then plot = "0"
end if
position (29,1)
printf(SCREEN, "Y=%s", {plot})

--Determine the Scale for graph (how much space between whole numbers)
Scale = {300 / (xmax[2] - xmin[2]) , 300 / (ymax[2] - ymin[2])}
--Determine the Center for graphing
Center = {(0-xmin[2])/(xmax[2]-xmin[2])*300+250, (0-ymin[2])/(ymax[2]-ymin[2])*-300+350}

-- Draw the window
polygon(BLUE, 0, {{250, 50}, {550, 50}, {550, 350}, {250, 350}})

--Draw origin
draw_line(GRAY, {{250, Center[2]}, {550, Center[2]}}) --X+
draw_line(GRAY, {{Center[1], 350}, {Center[1], 50}})  --Y+

--Draw the whole number ticks - X
for X1 = 250 to 550 by Scale[1]*stepx[2] do
   draw_line(WHITE, {{X1, Center[2]-2}, {X1, Center[2]+2}}) --Horz
end for

--Draw the whole number ticks - Y
for Y1 = 50 to 350 by Scale[2]*stepy[2] do
   draw_line(WHITE, {{Center[1]-2, Y1}, {Center[1]+2, Y1}}) --Vert
end for
end procedure

procedure draw_it()
   plotbk = plot     --Save the data in plot as the parse function destroys
                     --it.  This is for redawing after changing any option.
   for loop = xmin[2] to xmax[2] by detail[2] do
      prevX = X      --Save old values so a nice line can be made.
      prevY = Y
      X = loop
      Y = parse( plotbk )  --Determine Y using parse function.
      if Y < ymin[2] then Y = ymin[2]  --if value out of bounds then modify
      end if                         
      if Y > ymax[2] then Y = ymax[2]
      end if                         
      if X > -detail[2]/10 and X < detail[2]/10 then  --This is a dirty way
         position (1,1)                               --to find the intercepts
         printf(SCREEN, "Y-Int=%f", Y)                --Any ideas?
      end if                                          --Seems to work.
      if Y > -detail[2]/10 and Y < detail[2]/10 then
         position (count,1)
         count = count + 1
         if count = 20 then count = 3
         end if
         printf(SCREEN, "X-Int=%f", X)
      end if
      --plot the graph.
      draw_line (YELLOW, {{prevX*Scale[1]+Center[1], -prevY*Scale[2]+Center[2]}, {X*Scale[1]+Center[1], -Y*Scale[2]+Center[2]}})
   end for
end procedure

   initial()   --Draw the default graph.
   draw_it()
   initial()   --Second initial() is needed to redraw the axis.  Take it out
               --and you get a ugly looking thing.
while QUIT != 0 do   --Main loop.
   position(1,40)
   puts(SCREEN, "Function Line plotter V0.5b")
   position(26,1)
   puts(SCREEN, "Valid commands:xmin xmax ymin ymax stepx stepy plot detail quit\n")
   position(24,1)
   puts(SCREEN, "Enter command:")
   Command = gets(KEYBOARD)
   Command = Command[1..length(Command)-1]
   QUIT = compare(Command, "quit")
   if compare(Command, "xmin") = 0 then
      position(25,1)
      puts(SCREEN, "xmin:")
      xmin = ""
      xmin = get(KEYBOARD)
      count = 3            --Reset postion counter.
      clear_screen()
      initial()
      draw_it()
      initial()
   elsif compare(Command, "xmax") = 0 then
      position(25,1)
      puts(SCREEN, "xmax:")
      xmax = ""
      xmax = get(KEYBOARD)
      count = 3            --Reset postion counter.
      clear_screen()
      initial()
      draw_it()
      initial()
   elsif compare(Command, "ymax") = 0 then
      position(25,1)
      puts(SCREEN, "ymax:")
      ymax = ""
      ymax = get(KEYBOARD)
      count = 3            --Reset postion counter.
      clear_screen()
      initial()
      draw_it()
      initial()
   elsif compare(Command, "ymin") = 0 then
      position(25,1)
      puts(SCREEN, "ymin:")
      ymin = ""
      ymin = get(KEYBOARD)
      count = 3            --Reset postion counter.
      clear_screen()
      initial()
      draw_it()
      initial()
   elsif compare(Command, "stepx") = 0 then
      position(25,1)
      puts(SCREEN, "stepx:")
      stepx = ""
      stepx = get(KEYBOARD)
      count = 3            --Reset postion counter.
      clear_screen()
      initial()
      draw_it()
      initial()
   elsif compare(Command, "stepy") = 0 then
      position(25,1)
      puts(SCREEN, "stepy:")
      stepy = ""
      stepy = get(KEYBOARD)
      count = 3            --Reset postion counter.
      clear_screen()
      initial()
      draw_it()
      initial()
   elsif compare(Command, "plot") = 0 then
      position(25,1)
      puts(SCREEN, "Y=")
      plot = gets(KEYBOARD)
      count = 3            --Reset postion counter.
      clear_screen()
      initial()
      draw_it()
      initial()
   elsif compare(Command, "detail") = 0 then
      position(25,1)
      puts(SCREEN, "detail:")
      detail = get(KEYBOARD)
      count = 3            --Reset postion counter.
      clear_screen()
      initial()
      draw_it()
      initial()
   end if
end while

tmp = graphics_mode(3)     --Go back to a text mode so the prompt is readable
without warning            --ignore warnings
